import javax.swing.*;
import java.awt.*;
import java.awt.event.*;


/**
 * Dialog for letting the user choose what features to use.
 */
public class FeatureSelectionDialog extends JDialog implements ActionListener{
	boolean warpAlign=true, blend=true, biInterp=true;
	boolean hitOK=false;
	
	/**
	 * Constructor:
	 * @param f Frame this dialog belongs to.
	 * @param showAlignAndBlend true to show align and blend options, false otherwise.
	 */
	public FeatureSelectionDialog(Frame f, boolean showAlignAndBlend){
		super(f, "Choose the features to use", true);
		
		// create the checkboxes for feature options.
		JPanel jp=new JPanel();
		JCheckBox cb;
		if(showAlignAndBlend){
			cb=new JCheckBox("Warp align", true);
			cb.setActionCommand("walign");
			cb.addActionListener(this);
			jp.add(cb);
		
			cb=new JCheckBox("Image blending", true);
			cb.setActionCommand("blend");
			cb.addActionListener(this);
			jp.add(cb);
		}
		
		cb=new JCheckBox("Bilinear interpolation", true);
		cb.setActionCommand("binterp");
		cb.addActionListener(this);
		jp.add(cb);
		
		getContentPane().add(jp, BorderLayout.CENTER);
		
		// Create the 2 buttons allowing the user to confirm the input or cancel.
		jp=new JPanel();
		JButton b=new JButton("OK");
		b.addActionListener(this);
		jp.add(b);
		
		b=new JButton("Cancel");
		b.addActionListener(this);
		jp.add(b);
		getContentPane().add(jp, BorderLayout.SOUTH);
		
		setLocation(f.getWidth()/2-100, f.getHeight()/2-50);
		pack();
		setVisible(true);
	}
	
	/**
	 * Handles click events for the checkboxes and buttons.
	 * @param e Event object describing the event.
	 */
	public void actionPerformed(ActionEvent e){
		String c=e.getActionCommand();
		if(c.equals("walign")){
			warpAlign=!warpAlign;
		}else if(c.equals("blend")){
			blend=!blend;
		}else if(c.equals("binterp")){
			biInterp=!biInterp;
		}else if(c.equals("OK")){
			hitOK=true;
			dispose();
		}else if(c.equals("Cancel")){
			dispose();
		}
	}
}