import javax.swing.*;
import javax.swing.event.*;
import java.awt.event.*;
import java.awt.image.BufferedImage;
import java.awt.*;
import java.util.*;

/** Color prompt frame class. */
public class ForegroundExtractor extends JDialog {
	ColorPanel cp; //Panel with the colors shown.

	ButtonPanel bp; //Panel for buttons.

	BufferedImage image; //Original image.
	
	boolean hitSetForeground=true; // Did the user set the foreground.

	int aMaskDim = 3; // blend mask dimension.

	/** Constructor:
	 * @param ip The main application object.
	 */
	public ForegroundExtractor(Laf icApp, BufferedImage img) {
		super(icApp);
		setTitle("Copy foreground");
		setModal(true);
		image=img;

		getContentPane().setLayout(new BorderLayout());
		JPanel p=new JPanel();
		p.setLayout(new FlowLayout(FlowLayout.CENTER));
		p.add((cp = new ColorPanel()));
		getContentPane().add(p);
		
		getContentPane().add((bp = new ButtonPanel()), BorderLayout.SOUTH);

		pack();

		//Set the window's location.
		setLocation((icApp.getWidth()-getWidth())/2, (icApp.getHeight()-getHeight())/2);
		setVisible(true);
	}

	/** Creates the matte using the color and color range the user picked.
	 * @param color The color user picked for the matte.
	 * @param range The deviation range user picked for matte color.
	 */
	public BufferedImage setForegroundImage(BufferedImage img, int color, int range) {
		BufferedImage fg;
		int w = img.getWidth(null), h = img.getHeight(null);
		int r = (color >> 16) & 0xff;
		int g = (color >> 8) & 0xff;
		int b = (color) & 0xff;

		BufferedImage fgtmp =
			new BufferedImage(w, h, BufferedImage.TYPE_INT_ARGB);

		// The biggest color component. 0-red, 1-green, 2-blue
		int lead = 0;
		if (g >= r && g >= b)
			lead = 1;
		else if (b >= r && b >= g)
			lead = 2;

		int minX=w, minY=h, maxX=0, maxY=0;
		for (int y = 0; y < h; y++) {
			for (int x = 0; x < w; x++) {
				int p = img.getRGB(x, y);
				int alpha = (p >> 24) & 0xff;
				int red = (p >> 16) & 0xff;
				int green = (p >> 8) & 0xff;
				int blue = (p) & 0xff;

				// The color components are within range.
				if (r - range < red
					&& red < r + range
					&& g - range < green
					&& green < g + range
					&& b - range < blue
					&& blue < b + range) {
					// See if other color components are less than the lead.
					if (lead == 0 && red >= green && red >= blue)
						alpha = 0;
					else if (lead == 1 && green >= red && green >= blue)
						alpha = 0;
					else if (lead == 2 && blue >= red && blue >= green)
						alpha = 0;
				}
				p = (alpha << 24) + (red << 16) + (green << 8) + blue;
				if(alpha>0){
					if(x<minX) minX=x;
					if(x>maxX) maxX=x;
					if(y<minY) minY=y;
					if(y>maxY) maxY=y;
				}
				fgtmp.setRGB(x, y, p);
			}
		}
		return fgtmp;
	}
	
	/**
	 * Extracts the foregound of the image.
	 * @return Foreground of the image.
	 */
	public BufferedImage extractForeground(){
		BufferedImage fgtmp=setForegroundImage(image, cp.color, bp.range);
		int w=fgtmp.getWidth(), h=fgtmp.getHeight(), minX=w, minY=h, maxX=0, maxY=0;
		// find mins and maxs.
		for (int y = 0; y < h; y++) {
			for (int x = 0; x < w; x++) {
				int p = fgtmp.getRGB(x, y);
				int alpha = (p >> 24) & 0xff;
				if(alpha>0){
					if(x<minX) minX=x;
					if(x>maxX) maxX=x;
					if(y<minY) minY=y;
					if(y>maxY) maxY=y;
				}
			}
		}
		
		// Create alpha mask so that edges of the matte will be smooth.
		double sum = 0; // Total of the values in atmp.
		double[] atmp = new double[aMaskDim];
		// Compute the 1D mask (middle element greatest).
		for (int i = 0; i < aMaskDim; i++) {
			atmp[i] = 1 - (double) Math.abs(aMaskDim / 2 - i) * 2 / aMaskDim;
			sum += atmp[i];
		}

		// Compute the 2D mask for the weighted avg from the 1D mask.
		double[][] alphaMask = new double[aMaskDim][aMaskDim];
		for (int i = 0; i < aMaskDim; i++) {
			for (int j = 0; j < aMaskDim; j++) {
				alphaMask[i][j] = atmp[i] * atmp[j] / sum / sum;
			}
		}

		// Foreground, resized to fit the visible matte.
		BufferedImage fg=new BufferedImage(maxX-minX+aMaskDim, maxY-minY+aMaskDim, BufferedImage.TYPE_INT_ARGB);
		int ax, ay, p2, alpha;
		// Apply the mask to the matte.
		for (int y = Math.max(0, minY-aMaskDim/2); y <= Math.min(h-1, maxY+aMaskDim/2); y++) {
			for (int x = Math.max(0, minX-aMaskDim/2); x <= Math.min(w-1, maxX+aMaskDim/2); x++) {
				double alphaAvg = 0;
				// calculate the weighted avg around x, y sing the mask.
				for (int i = 0; i < aMaskDim; i++) {
					for (int j = 0; j < aMaskDim; j++) {
						ax=Math.max(0, Math.min(w-1, x + i - aMaskDim / 2));
						ay=Math.max(0, Math.min(h-1, y + j - aMaskDim / 2));

						p2 =fgtmp.getRGB(ax, ay);
						alpha = (p2 >> 24) & 0xff;
						alphaAvg += alpha * alphaMask[j][i];
						Paint.getProgress().getSetCurrentProgress(
							Paint.getProgress().getSetCurrentProgress(-1)+1);
					}
				}
				fg.setRGB(
					x-minX+aMaskDim/2,
					y-minY+aMaskDim/2,
					((int) Math.round(alphaAvg) << 24)
						+ (fgtmp.getRGB(x, y) & 0x00ffffff));
			}
		}
		
		return fg;
	}
	
	/**
	 * Estimate the progress of the extraction.
	 * @return Total progress of the extraction.
	 */
	public int estimateExtractForegroundProgress(){
		return image.getWidth()*image.getHeight()*aMaskDim*aMaskDim;
	}

	/** Panel for showing the colors to the user. */

	private class ColorPanel extends JPanel implements MouseListener, MouseMotionListener{

		int color=0, range = 0, x=0, y=0, dragX, dragY;
		double zoom=100;
		BufferedImage prev;
		boolean changeColor=false;
		JPanel imgHolder;

		/** Constructor:
		 */
		public ColorPanel() {
			setLayout(null);
			// set minimum dimesnions.
			Dimension size = new Dimension(100, 100);
			this.setMinimumSize(size);
			this.setPreferredSize(size);
			x=0;
			y=0;
			addMouseListener(this);
			addMouseMotionListener(this);
			setZoom(100);
		}

		/** Paints the panel.
		 * @param g The Graphics object to paint in.
		 */
		public void paintComponent(Graphics g) {
			super.paintComponent(g);
			// new image to the zoomed out dimensions.
			BufferedImage im=new BufferedImage((int)(image.getWidth()*zoom/100), (int)(image.getHeight()*zoom/100), BufferedImage.TYPE_INT_ARGB);
			// draw the orinal image into it.
			im.createGraphics().drawImage(image,
				0, 0, im.getWidth()-1, im.getHeight()-1, 0, 0, image.getWidth()-1,
				image.getHeight()-1, null);
			// new image for the preview.
			prev=new BufferedImage(100, 100, BufferedImage.TYPE_INT_ARGB);
			prev.createGraphics().drawImage(im, x, y, null);
			// extract the foreground on the preview.
			prev=setForegroundImage(prev, color, range);
			g.drawImage(prev, 0, 0, null);
			
			// border
			Color c=g.getColor();
			g.setColor(new Color(80, 80, 80));
			g.drawRect(0, 0, 99, 99);
			g.setColor(c);
		}

		/** Changes the color and range for this panel to show.
		 * @param c The cnew color to show.
		 * @param r The new deviation from the color to use.
		 */
		public void setColor(int c, int r) {
			color = c;
			range = r;
			repaint();
		}
		
		/**
		 * Sets zoom amount.
		 * @param zoom Zoom amount in percent.
		 */
		public void setZoom(int zoom){
			this.zoom=zoom;
			repaint();
		}

		/* (non-Javadoc)
		 * @see java.awt.event.MouseMotionListener#mouseDragged(java.awt.event.MouseEvent)
		 */
		public void mouseDragged(MouseEvent e) {
			x+=e.getX()-dragX;
			y+=e.getY()-dragY;
			dragX=e.getX();
			dragY=e.getY();
			repaint();
		}

		/* (non-Javadoc)
		 * @see java.awt.event.MouseMotionListener#mouseMoved(java.awt.event.MouseEvent)
		 */
		public void mouseMoved(MouseEvent e) {
		}

		/* (non-Javadoc)
		 * @see java.awt.event.MouseListener#mouseClicked(java.awt.event.MouseEvent)
		 */
		public void mouseClicked(MouseEvent e) {
			if(prev!=null){
				setColor(prev.getRGB(e.getX(), e.getY()), bp.range);
			}
		}

		/* (non-Javadoc)
		 * @see java.awt.event.MouseListener#mouseEntered(java.awt.event.MouseEvent)
		 */
		public void mouseEntered(MouseEvent e) {
			setCursor(Laf.getCursor("dropper"));
		}

		/* (non-Javadoc)
		 * @see java.awt.event.MouseListener#mouseExited(java.awt.event.MouseEvent)
		 */
		public void mouseExited(MouseEvent e) {
			setCursor(Laf.getCursor("default"));
			
		}

		/* (non-Javadoc)
		 * @see java.awt.event.MouseListener#mousePressed(java.awt.event.MouseEvent)
		 */
		public void mousePressed(MouseEvent e) {
			dragX=e.getX();
			dragY=e.getY();
			setCursor(Laf.getCursor("hand"));
		}

		/* (non-Javadoc)
		 * @see java.awt.event.MouseListener#mouseReleased(java.awt.event.MouseEvent)
		 */
		public void mouseReleased(MouseEvent e) {
			setCursor(Laf.getCursor("dropper"));
		}
	}

	/** Panel for holding the buttons to allow the user to set the range. */

	private class ButtonPanel extends JPanel implements ActionListener, ChangeListener {
		JSlider s, zoom; // User input

		int range = 0;

		/** Constructor:
		 */
		public ButtonPanel() {
			setLayout(new GridLayout(3, 2));

			add(new JLabel("Zoom:"));

			// Create the zoom slider.
			zoom = new JSlider(JSlider.HORIZONTAL, 1, 100, 100);
			Hashtable d = new Hashtable();
			d.put(new Integer(1), new JLabel("1"));
			d.put(new Integer(50), new JLabel("50"));
			d.put(new Integer(100), new JLabel("100"));
			zoom.setLabelTable(d);
			zoom.setPaintLabels(true);
			add(zoom);
			zoom.addChangeListener(new ChangeListener(){
				public void stateChanged(ChangeEvent e){
					cp.setZoom(zoom.getValue());
				}
			});

			add(new JLabel("Tolerance:"));

			// Create the tolerance slider.
			s = new JSlider(JSlider.HORIZONTAL, 0, 255, range);
			d = new Hashtable();
			d.put(new Integer(0), new JLabel("0"));
			d.put(new Integer(128), new JLabel("128"));
			d.put(new Integer(255), new JLabel("255"));
			s.setLabelTable(d);
			s.setPaintLabels(true);
			add(s);
			s.addChangeListener(this);

			// Choose the color and range button.
			JButton b = new JButton("Copy foreground");
			b.addActionListener(this);
			add(b);

			// Choose the color and range button.
			b = new JButton("Cancel");
			b.addActionListener(this);
			add(b);
			setPreferredSize(getMinimumSize());
		}

		/** Action event handler.
		 * @param e ActionEvent object describing the event.
		 */
		public void actionPerformed(ActionEvent e) {
			if(e.getActionCommand().equals("Cancel")){
				hitSetForeground=false;
			}
			// Sets the color and eviation.
			range = s.getValue();
			dispose();
		}

		/** Slider change event handler.
		 * @param e ChangeEvent object describing the event.
		 */
		public void stateChanged(ChangeEvent e) {
			range = s.getValue(); // Set tolerance
			cp.setColor(cp.color, range); // Show colors.
		}
	}
}