import java.awt.*;
import java.awt.image.*;
import javax.swing.*;

/**
 * This class is for image manipulating utilities.
 */
public class Paint{
	private static Progress prog=new Progress();	// holds progress of an operation.
	// arguments for an operation.
	private static java.util.Hashtable args=new java.util.Hashtable();
	// result from the operation.
	private static Object result;
	
	/**
	 * Returns the object with the progress of an operation.
	 * @return The object with the progress of an operation.
	 */
	public static Progress getProgress(){
		return prog;
	}

	/**
	 * Warps the image according to the 8 parameters given and sets the rc parameters, such
	 * that they match pixels in the warped image.
	 * @param s Source image.
	 * @param uv1 Source pixel 1.
	 * @param uv2 Source pixel 2.
	 * @param uv3 Source pixel 3.
	 * @param uv4 Source pixel 4.
	 * @param rc1 Target pixel 1.
	 * @param rc2 Target pixel 2.
	 * @param rc3 Target pixel 3.
	 * @param rc4 Target pixel 4.
	 * @return Warped image.
	 */
	public static BufferedImage getWarp(
		BufferedImage s,
		XYPair uv1,
		XYPair uv2,
		XYPair uv3,
		XYPair uv4,
		XYPair rc1,
		XYPair rc2,
		XYPair rc3,
		XYPair rc4,
		boolean [] features) {
		args.clear();
		args.put("s", s);
		args.put("uv1", uv1);
		args.put("uv2", uv2);
		args.put("uv3", uv3);
		args.put("uv4", uv4);
		args.put("rc1", rc1);
		args.put("rc2", rc2);
		args.put("rc3", rc3);
		args.put("rc4", rc4);
		args.put("features", features);
		if(prog.getSetCurrentProgress(-1)>0)
			return getWarp();
		
		
		prog.getSetCurrentProgress(1);
		Dimension d=estimateGetWarpProgress(
			s,
			uv1,
			uv2,
			uv3,
			uv4,
			new XYPair(rc1.x, rc1.y),
			new XYPair(rc2.x, rc2.y),
			new XYPair(rc3.x, rc3.y),
			new XYPair(rc4.x, rc4.y));
		
		Thread t=new Thread(new Runnable(){
			public void run(){
				result=getWarp();
				prog.getSetCurrentProgress(0);
			}
		});
		t.start();
		Dialogs.showProgressBar(prog, d.width*d.height);
		return (BufferedImage)result;
	}

	/**
	 * Warps the image according to the 8 parameters given and sets the rc parameters, such
	 * that they match pixels in the warped image.
	 * @return Warped image.
	 */
	private static BufferedImage getWarp() {
		BufferedImage s=(BufferedImage)args.get("s");
		XYPair uv1=(XYPair)args.get("uv1");
		XYPair uv2=(XYPair)args.get("uv2");
		XYPair uv3=(XYPair)args.get("uv3");
		XYPair uv4=(XYPair)args.get("uv4");
		XYPair rc1=(XYPair)args.get("rc1");
		XYPair rc2=(XYPair)args.get("rc2");
		XYPair rc3=(XYPair)args.get("rc3");
		XYPair rc4=(XYPair)args.get("rc4");
		boolean [] features=(boolean [])args.get("features");
		BufferedImage res;
		TextureMap tm;
		
		// get the boundaries of the warped image by switching the source and the target.
		tm =new TextureMap(
			rc1.x,rc1.y,rc2.x,rc2.y,rc3.x,rc3.y,rc4.x,rc4.y,
			uv1.x,uv1.y,uv2.x,uv2.y,uv3.x,uv3.y,uv4.x,uv4.y);
		int minX, minY, maxX, maxY, tmpX, tmpY;
		int sw = s.getWidth(null), sh = s.getHeight(null);

		// find the dimensions needed for the whole warped image.
		minX = maxX = (int) tm.texU(0, 0);
		minY = maxY = (int) tm.texV(0, 0);

		if ((tmpX = (int) tm.texU(sw - 1, 0)) > maxX)
			maxX = tmpX;
		else if (tmpX < minX)
			minX = tmpX;

		if ((tmpY = (int) tm.texV(sw - 1, 0)) > maxY)
			maxY = tmpY;
		else if (tmpY < minY)
			minY = tmpY;

		if ((tmpX = (int) tm.texU(0, sh - 1)) > maxX)
			maxX = tmpX;
		else if (tmpX < minX)
			minX = tmpX;

		if ((tmpY = (int) tm.texV(0, sh - 1)) > maxY)
			maxY = tmpY;
		else if (tmpY < minY)
			minY = tmpY;

		if ((tmpX = (int) tm.texU(sw - 1, sh - 1)) > maxX)
			maxX = tmpX;
		else if (tmpX < minX)
			minX = tmpX;

		if ((tmpY = (int) tm.texV(sw - 1, sh - 1)) > maxY)
			maxY = tmpY;
		else if (tmpY < minY)
			minY = tmpY;

		// offset the target points to account for the different size of the warp image.
		rc1.x -= minX;
		rc1.y -= minY;
		rc2.x -= minX;
		rc2.y -= minY;
		rc3.x -= minX;
		rc3.y -= minY;
		rc4.x -= minX;
		rc4.y -= minY;

		res =
			new BufferedImage(
				maxX - minX + 1,
				maxY - minY + 1,
				BufferedImage.TYPE_INT_ARGB);
		// create texturizer offsetting the points to account for resized image.
		tm =new TextureMap(
			uv1.x,uv1.y,uv2.x,uv2.y,uv3.x,uv3.y,uv4.x,uv4.y,
			rc1.x,rc1.y,rc2.x,rc2.y,rc3.x,rc3.y,rc4.x,rc4.y);

		// set the warped pixels.
		for (int i = 0; i < maxY - minY + 1; i++) {
			for (int j = 0; j < maxX - minX + 1; j++) {
				int x1=(int)tm.texU(j,i);
				int x2=x1+1;
				int y1=(int)tm.texV(j,i);
				int y2=y1+1;
				
				int pixel=0;
				if(features[2])
					pixel=biInt(s, x1, x2, y1, y2, tm.texU(j,i), tm.texV(j,i));
				else if(0<=x1 && x1<sw && 0<=y1 && y1<sh)
					pixel=s.getRGB(x1, y1);
				res.setRGB(j, i, pixel);
				prog.getSetCurrentProgress(prog.getSetCurrentProgress(-1)+1);
			}
		}

		return res;
	}
	
	/**
	 * Returns an estimate of how long getWarp method may take. Does the offset to the
	 * target points.
	 * @param s Source image.
	 * @param uv1 Source pixel 1.
	 * @param uv2 Source pixel 2.
	 * @param uv3 Source pixel 3.
	 * @param uv4 Source pixel 4.
	 * @param rc1 Target pixel 1.
	 * @param rc2 Target pixel 2.
	 * @param rc3 Target pixel 3.
	 * @param rc4 Target pixel 4.
	 * @return Dimension of the warped image, can get the estimate by multiplying the dims.
	 */
	public static Dimension estimateGetWarpProgress(
		BufferedImage s,
		XYPair uv1,
		XYPair uv2,
		XYPair uv3,
		XYPair uv4,
		XYPair rc1,
		XYPair rc2,
		XYPair rc3,
		XYPair rc4) {
		BufferedImage res;
		TextureMap tm;
		// get the boundaries of the warped image by switching the source and the target.
		tm =new TextureMap(
			rc1.x,rc1.y,rc2.x,rc2.y,rc3.x,rc3.y,rc4.x,rc4.y,
			uv1.x,uv1.y,uv2.x,uv2.y,uv3.x,uv3.y,uv4.x,uv4.y);
		int minX, minY, maxX, maxY, tmpX, tmpY;
		int sw = s.getWidth(null), sh = s.getHeight(null);

		// find the dimensions needed for the whole warped image.
		minX = maxX = (int) tm.texU(0, 0);
		minY = maxY = (int) tm.texV(0, 0);

		if ((tmpX = (int) tm.texU(sw - 1, 0)) > maxX)
			maxX = tmpX;
		else if (tmpX < minX)
			minX = tmpX;

		if ((tmpY = (int) tm.texV(sw - 1, 0)) > maxY)
			maxY = tmpY;
		else if (tmpY < minY)
			minY = tmpY;

		if ((tmpX = (int) tm.texU(0, sh - 1)) > maxX)
			maxX = tmpX;
		else if (tmpX < minX)
			minX = tmpX;

		if ((tmpY = (int) tm.texV(0, sh - 1)) > maxY)
			maxY = tmpY;
		else if (tmpY < minY)
			minY = tmpY;

		if ((tmpX = (int) tm.texU(sw - 1, sh - 1)) > maxX)
			maxX = tmpX;
		else if (tmpX < minX)
			minX = tmpX;

		if ((tmpY = (int) tm.texV(sw - 1, sh - 1)) > maxY)
			maxY = tmpY;
		else if (tmpY < minY)
			minY = tmpY;	
		//offset the target points to account for the different size of the warp image.
		rc1.x -= minX;
		rc1.y -= minY;
		rc2.x -= minX;
		rc2.y -= minY;
		rc3.x -= minX;
		rc3.y -= minY;
		rc4.x -= minX;
		rc4.y -= minY;
		return new Dimension(maxX-minX, maxY-minY);
	}
	
	/**
	 * Adjusts the source points for an approximate best warp alignment.
	 * @param s The source image.
	 * @param t The target image.
	 * @param uv1 Source point 1.
	 * @param uv2 Source point 2.
	 * @param uv3 Source point 3.
	 * @param uv4 Source point 4.
	 * @param rc1 Target point 1.
	 * @param rc2 Target point 2.
	 * @param rc3 Target point 3.
	 * @param rc4 Target point 4.
	 */
	private static void getGoodWarp(
		BufferedImage s,
		BufferedImage t,
		XYPair uv1,
		XYPair uv2,
		XYPair uv3,
		XYPair uv4,
		XYPair rc1,
		XYPair rc2,
		XYPair rc3,
		XYPair rc4,
		boolean [] features){
		
		TextureMap tm;
		XYPair uv21=new XYPair(uv1.x, uv1.y), uv22=new XYPair(uv2.x, uv2.y),
			uv23=new XYPair(uv3.x, uv3.y), uv24=new XYPair(uv4.x, uv4.y);
		double error, minErr;
		// the steps to take away from a point.
		int [][] steps={{0,0},{-1, -1},{-1, 1},{1, -1},{1, 1},{0, -1},{0, 1},{1, 0},{-1, 0}};
		
		// initial error with no adjustment.
		minErr=error=getError(s, t, uv1,uv2,uv3,uv4,rc1,rc2,rc3,rc4, features);
		int iter=0;
		while(iter++<5){
			// go through every point and every adgust step.
			for(int i=0; i<steps.length; i++){
				uv1.x+=steps[i][0];
				uv1.y+=steps[i][1];
				prog.getSetCurrentProgress(prog.getSetCurrentProgress(-1)+1);
				try{
					error=getError(s, t, uv1,uv2,uv3,uv4,rc1,rc2,rc3,rc4, features);
					if(error<minErr){
						// smaller error so store these points.
						uv21.x=uv1.x;
						uv21.y=uv1.y;
						minErr=error;
					}
				}catch(IllegalArgumentException e){
				}
				uv1.x-=steps[i][0];
				uv1.y-=steps[i][1];
			}
			// change the source points to the adjusted points.
			uv1.x=uv21.x;
			uv1.y=uv21.y;
			// go through every point and every adgust step.
			for(int i=0; i<steps.length; i++){
				uv2.x+=steps[i][0];
				uv2.y+=steps[i][1];
				prog.getSetCurrentProgress(prog.getSetCurrentProgress(-1)+1);
				try{
					error=getError(s, t, uv1,uv2,uv3,uv4,rc1,rc2,rc3,rc4, features);
					if(error<minErr){
						// smaller error so store these points.
						uv22.x=uv2.x;
						uv22.y=uv2.y;
						minErr=error;
					}
				}catch(IllegalArgumentException e){
				}
				uv2.x-=steps[i][0];
				uv2.y-=steps[i][1];
			}
			// change the source points to the adjusted points.
			uv2.x=uv22.x;
			uv2.y=uv22.y;
			// go through every point and every adgust step.
			for(int i=0; i<steps.length; i++){
				uv3.x+=steps[i][0];
				uv3.y+=steps[i][1];
				prog.getSetCurrentProgress(prog.getSetCurrentProgress(-1)+1);
				try{
					error=getError(s, t, uv1,uv2,uv3,uv4,rc1,rc2,rc3,rc4, features);
					if(error<minErr){
						// smaller error so store these points.
						uv23.x=uv3.x;
						uv23.y=uv3.y;
						minErr=error;
					}
				}catch(IllegalArgumentException e){
				}
				uv3.x-=steps[i][0];
				uv3.y-=steps[i][1];
			}
			// change the source points to the adjusted points.
			uv3.x=uv23.x;
			uv3.y=uv23.y;
			// go through every point and every adgust step.
			for(int i=0; i<steps.length; i++){
				uv4.x+=steps[i][0];
				uv4.y+=steps[i][1];
				prog.getSetCurrentProgress(prog.getSetCurrentProgress(-1)+1);
				try{
					error=getError(s, t, uv1,uv2,uv3,uv4,rc1,rc2,rc3,rc4, features);
					if(error<minErr){
						// smaller error so store these points.
						uv24.x=uv4.x;
						uv24.y=uv4.y;
						minErr=error;
					}
				}catch(IllegalArgumentException e){
				}
				uv4.x-=steps[i][0];
				uv4.y-=steps[i][1];
			}
			// change the source points to the adjusted points.
			uv4.x=uv24.x;
			uv4.y=uv24.y;
		}
	}
	
	/**
	 * Estimates the progress of getGoodWarp.
	 * @param s The source image.
	 * @param t The target image.
	 * @param uv1 Source point 1.
	 * @param uv2 Source point 2.
	 * @param uv3 Source point 3.
	 * @param uv4 Source point 4.
	 * @param rc1 Target point 1.
	 * @param rc2 Target point 2.
	 * @param rc3 Target point 3.
	 * @param rc4 Target point 4.
	 * @return Estimated total progress for getGoodWarp.
	 */
	private static int estimateGetGoodWarpProgress(
		BufferedImage s,
		BufferedImage t,
		XYPair uv1,
		XYPair uv2,
		XYPair uv3,
		XYPair uv4,
		XYPair rc1,
		XYPair rc2,
		XYPair rc3,
		XYPair rc4){
		// the steps to take away from a point.
		int [][] steps={{0,0},{-1, -1},{-1, 1},{1, -1},{1, 1},{0, -1},{0, 1},{1, 0},{-1, 0}};
		return (int)steps.length*4*5*
			estimateGetErrorProgress(s,t,uv1,uv2,uv3,uv4,rc1,rc2,rc3,rc4);
	}
	
	/**
	 * Calculates the error between 2 images.
	 * @param s The source image.
	 * @param t The target image.
	 * @param uv1 Source point 1.
	 * @param uv2 Source point 2.
	 * @param uv3 Source point 3.
	 * @param uv4 Source point 4.
	 * @param rc1 Target point 1.
	 * @param rc2 Target point 2.
	 * @param rc3 Target point 3.
	 * @param rc4 Target point 4.
	 * @return Error between 2 images.
	 */
	private static double getError(BufferedImage s, BufferedImage t,
		XYPair uv1,
		XYPair uv2,
		XYPair uv3,
		XYPair uv4,
		XYPair rc1,
		XYPair rc2,
		XYPair rc3,
		XYPair rc4,
		boolean [] features){
		// get the boundaries of the warped image by switching the source and the target.
		TextureMap tm =new TextureMap(
			rc1.x,rc1.y,rc2.x,rc2.y,rc3.x,rc3.y,rc4.x,rc4.y,
			uv1.x,uv1.y,uv2.x,uv2.y,uv3.x,uv3.y,uv4.x,uv4.y);
		int minX=0, minY=0, maxX=t.getWidth(), maxY=t.getHeight(), tmpX, tmpY;
		int sw = s.getWidth(), sh = s.getHeight();

		// find the dimensions needed for the whole warped image.
		minX = maxX = (int) tm.texU(0, 0);
		minY = maxY = (int) tm.texV(0, 0);

		if ((tmpX = (int) tm.texU(sw - 1, 0)) > maxX)
			maxX = tmpX;
		else if (tmpX < minX)
			minX = tmpX;

		if ((tmpY = (int) tm.texV(sw - 1, 0)) > maxY)
			maxY = tmpY;
		else if (tmpY < minY)
			minY = tmpY;

		if ((tmpX = (int) tm.texU(0, sh - 1)) > maxX)
			maxX = tmpX;
		else if (tmpX < minX)
			minX = tmpX;

		if ((tmpY = (int) tm.texV(0, sh - 1)) > maxY)
			maxY = tmpY;
		else if (tmpY < minY)
			minY = tmpY;

		if ((tmpX = (int) tm.texU(sw - 1, sh - 1)) > maxX)
			maxX = tmpX;
		else if (tmpX < minX)
			minX = tmpX;

		if ((tmpY = (int) tm.texV(sw - 1, sh - 1)) > maxY)
			maxY = tmpY;
		else if (tmpY < minY)
			minY = tmpY;
					 
		tm=new TextureMap(
			uv1.x,uv1.y,uv2.x,uv2.y,uv3.x,uv3.y,uv4.x,uv4.y,
			rc1.x,rc1.y,rc2.x,rc2.y,rc3.x,rc3.y,rc4.x,rc4.y);
		double err=0, ta, sa;
		int x, y, p, tr, tg, tb, sr, sg, sb, intersectNum=0;
		// go through evry pixel and accumulate the error.
		for(int i=Math.max(minY, 0); i<Math.min(maxY+1, t.getHeight()); i++){
			for(int j=Math.max(minX, 0); j<Math.min(maxX+1, t.getWidth()); j++){
				x=(int)tm.texU(j, i);
				y=(int)tm.texV(j, i);
				
				int x1=x;
				int x2=x1+1;
				int y1=y;
				int y2=y1+1;
								
				p=t.getRGB(j, i);
				ta=(p>>24)&0xff;
				tr=(p>>16)&0xff;
				tg=(p>>8)&0xff;
				tb=p&0xff;
				// see if the source point is in the source image.
				if(x<0 || x>=s.getWidth() || y<0 || y>=s.getHeight() || p==0)
					continue;
				
				p=0;
				if(features[2])
					p = biInt(s, x1, x2, y1, y2, tm.texU(j,i), tm.texV(j,i));
				else if(0<=x && x<sw && 0<=y && y<sh)
					p=s.getRGB(x, y);

				// skip if the source pixel is blank.
				if(p==0)
					continue;
				sa=(p>>24)&0xff;
				sr=(p>>16)&0xff;
				sg=(p>>8)&0xff;
				sb=p&0xff;
				err+=(Math.pow(sa*sr/255-ta*tr/255, 2)+
					Math.pow(sa*sg/255-ta*tg/255, 2)+
					Math.pow(sa*sb/255-ta*tb/255, 2))/3;
					
				intersectNum++;
				prog.getSetCurrentProgress(prog.getSetCurrentProgress(-1)+1);
			}
		}
		return err/intersectNum;
	}
	
	/**
	 * Estimates the progress of getError.
	 * @param s The source image.
	 * @param t The target image.
	 * @param uv1 Source point 1.
	 * @param uv2 Source point 2.
	 * @param uv3 Source point 3.
	 * @param uv4 Source point 4.
	 * @param rc1 Target point 1.
	 * @param rc2 Target point 2.
	 * @param rc3 Target point 3.
	 * @param rc4 Target point 4.
	 * @return Estimated total progress of getError.
	 */
	private static int estimateGetErrorProgress(BufferedImage s, BufferedImage t,
		XYPair uv1,
		XYPair uv2,
		XYPair uv3,
		XYPair uv4,
		XYPair rc1,
		XYPair rc2,
		XYPair rc3,
		XYPair rc4){
		// get the boundaries of the warped image by switching the source and the target.
		TextureMap tm =new TextureMap(
			rc1.x,rc1.y,rc2.x,rc2.y,rc3.x,rc3.y,rc4.x,rc4.y,
			uv1.x,uv1.y,uv2.x,uv2.y,uv3.x,uv3.y,uv4.x,uv4.y);
		int minX=0, minY=0, maxX=t.getWidth(), maxY=t.getHeight(), tmpX, tmpY;
		int sw = s.getWidth(), sh = s.getHeight();

		// find the dimensions needed for the whole warped image.
		minX = maxX = (int) tm.texU(0, 0);
		minY = maxY = (int) tm.texV(0, 0);

		if ((tmpX = (int) tm.texU(sw - 1, 0)) > maxX)
			maxX = tmpX;
		else if (tmpX < minX)
			minX = tmpX;

		if ((tmpY = (int) tm.texV(sw - 1, 0)) > maxY)
			maxY = tmpY;
		else if (tmpY < minY)
			minY = tmpY;

		if ((tmpX = (int) tm.texU(0, sh - 1)) > maxX)
			maxX = tmpX;
		else if (tmpX < minX)
			minX = tmpX;

		if ((tmpY = (int) tm.texV(0, sh - 1)) > maxY)
			maxY = tmpY;
		else if (tmpY < minY)
			minY = tmpY;

		if ((tmpX = (int) tm.texU(sw - 1, sh - 1)) > maxX)
			maxX = tmpX;
		else if (tmpX < minX)
			minX = tmpX;

		if ((tmpY = (int) tm.texV(sw - 1, sh - 1)) > maxY)
			maxY = tmpY;
		else if (tmpY < minY)
			minY = tmpY;
			
		return (Math.min(maxY+1, t.getHeight())-Math.max(minY, 0))*
			(Math.min(maxX+1, t.getWidth())-Math.max(minX, 0)); 
	}
	
	/**
	 * Returns a mosaic of the source and the target with a blend of pixels at their intersection.
	 * @param s The source image.
	 * @param t The target image.
	 * @param uv1 Source point 1.
	 * @param uv2 Source point 2.
	 * @param uv3 Source point 3.
	 * @param uv4 Source point 4.
	 * @param rc1 Target point 1.
	 * @param rc2 Target point 2.
	 * @param rc3 Target point 3.
	 * @param rc4 Target point 4.
	 * @return Mosaiced image.
	 */
	public static BufferedImage createMosaic(BufferedImage s, BufferedImage t, XYPair uv1, XYPair uv2, XYPair uv3, XYPair uv4,
		XYPair rc1, XYPair rc2, XYPair rc3, XYPair rc4, boolean [] features){
		args.clear();
		args.put("s", s);
		args.put("t", t);
		args.put("uv1", new XYPair(uv1.x, uv1.y));
		args.put("uv2", new XYPair(uv2.x, uv2.y));
		args.put("uv3", new XYPair(uv3.x, uv3.y));
		args.put("uv4", new XYPair(uv4.x, uv4.y));
		args.put("rc1", rc1);
		args.put("rc2", rc2);
		args.put("rc3", rc3);
		args.put("rc4", rc4);
		args.put("features", features);
		
		if(prog.getSetCurrentProgress(-1)>0)
			return createMosaic();
			
		prog.getSetCurrentProgress(1);
		int total=estimateCreateMosaicProgress(
			s,
			t,
			uv1,
			uv2,
			uv3,
			uv4,
			new XYPair(rc1.x, rc1.y),
			new XYPair(rc2.x, rc2.y),
			new XYPair(rc3.x, rc3.y),
			new XYPair(rc4.x, rc4.y), features);
		
		Thread mThread=new Thread(new Runnable(){
			public void run(){
				result=createMosaic();
				prog.getSetCurrentProgress(0);
			}
		});
		mThread.start();
		Dialogs.showProgressBar(prog, total);
		return (BufferedImage)result;
	}
	
	/**
	 * Returns a mosaic of the source and the target with a blend of pixels at their intersection.
	 * @return Mosaiced image.
	 */
	private static BufferedImage createMosaic(){
		BufferedImage s=(BufferedImage)args.get("s");
		BufferedImage t=(BufferedImage)args.get("t");
		XYPair uv1=(XYPair)args.get("uv1");
		XYPair uv2=(XYPair)args.get("uv2");
		XYPair uv3=(XYPair)args.get("uv3");
		XYPair uv4=(XYPair)args.get("uv4");
		XYPair rc1=(XYPair)args.get("rc1");
		XYPair rc2=(XYPair)args.get("rc2");
		XYPair rc3=(XYPair)args.get("rc3");
		XYPair rc4=(XYPair)args.get("rc4");
		boolean [] features=(boolean [])args.get("features");
		
		BufferedImage res, warp;
				
		XYPair rc21=new XYPair(rc1.x, rc1.y), rc22=new XYPair(rc2.x, rc2.y),
						rc23=new XYPair(rc3.x, rc3.y), rc24=new XYPair(rc4.x, rc4.y);
		
		if(features[0])
			getGoodWarp(s, t, uv1, uv2, uv3, uv4, rc21, rc22, rc23, rc24, features);
		warp=getWarp(s, uv1, uv2, uv3, uv4, rc21, rc22, rc23, rc24, features);
		
		int th=t.getHeight(), tw=t.getWidth(), wh=warp.getHeight(), ww=warp.getWidth();
		//the size of the intersection
		int intersectWid = Math.min(ww-rc21.x, tw-rc1.x)+Math.min(rc21.x, rc1.x),
		intersectHet = Math.min(wh-rc21.y, th-rc1.y)+Math.min(rc21.y,rc1.y);
		
		// calculate the width, height requirement for result
		// (add dims then subtract the size of the intersection).
		int wid=tw+ww-intersectWid,
			het=th+wh-intersectHet;
		
		res=new BufferedImage(wid, het, BufferedImage.TYPE_INT_ARGB);	
		int tx = Math.max(rc21.x-rc1.x, 0); //origin x of the target image
		int ty = Math.max(rc21.y-rc1.y, 0); //origin y of the target image
		int wx = Math.max(rc1.x-rc21.x, 0); //origin x of the warped image
		int wy = Math.max(rc1.y-rc21.y, 0); //origin y of the warped image
		
		// Bounded area of the intersection
		int xMin = Math.max(tx, wx); 
		int yMin = Math.max(ty, wy);
		int xMax = Math.max(tx, wx)+intersectWid;
		int yMax = Math.max(ty, wy)+intersectHet;
				
		// merge the warp and the traget.
		res.createGraphics().drawImage(t, tx, ty, null);
		res.createGraphics().drawImage(warp, wx, wy, null);
	
		if(features[1]){
			double [][] distt=getSmallestDist(t, xMin-tx, xMax-tx, yMin-ty, yMax-ty);
			double [][] distw=getSmallestDist(warp, xMin-wx, xMax-wx, yMin-wy, yMax-wy);
			// calculate the weighting within the intersected area and determine new intensities for those pixels
			for (int x = xMin; x < xMax; x++){
				for (int y = yMin; y < yMax; y++){
					int pixelt = t.getRGB(x-tx, y-ty);
					int alphat = (pixelt >> 24) & 0xff;
					int redt = (pixelt >> 16) & 0xff; 
					int greent = (pixelt >> 8) & 0xff;
					int bluet = (pixelt) & 0xff;
               
					int pixelr = res.getRGB(x, y);
					int alphar = (pixelr >> 24) & 0xff;
					int redr = (pixelr >> 16) & 0xff; 
					int greenr = (pixelr >> 8) & 0xff;
					int bluer = (pixelr) & 0xff;
               
					int alpha;
					int red;
					int green;
					int blue;
               
					// if the pixel has not been drawn, no need to calculate its weight
					if(pixelt!=0 && warp.getRGB(x-wx, y-wy)!=0){
						double dt=Math.pow(distt[y-yMin][x-xMin], 3), dw=Math.pow(distw[y-yMin][x-xMin], 3), weight=dt/(dt+dw);
						alpha = (int)(weight*alphat + (1-weight)*alphar);
						red = (int)(weight*redt + (1-weight)*redr);
						green = (int)(weight*greent + (1-weight)*greenr);
						blue = (int)(weight*bluet + (1-weight)*bluer);
		
						res.setRGB(x, y, (alpha<<24)+(red<<16)+(green<<8)+blue);
					}

					prog.getSetCurrentProgress(prog.getSetCurrentProgress(-1)+1);
				}
			}
		}
	
		return res;
	}
	
	/**
	 * Estimates the progresss of createMosaic.
	 * @param s The source image.
	 * @param t The target image.
	 * @param uv1 Source point 1.
	 * @param uv2 Source point 2.
	 * @param uv3 Source point 3.
	 * @param uv4 Source point 4.
	 * @param rc1 Target point 1.
	 * @param rc2 Target point 2.
	 * @param rc3 Target point 3.
	 * @param rc4 Target point 4.
	 * @return Estimated total progress of createMosaic.
	 */
	public static int estimateCreateMosaicProgress(BufferedImage s, BufferedImage t, XYPair uv1, XYPair uv2, XYPair uv3, XYPair uv4,
			XYPair rc1, XYPair rc2, XYPair rc3, XYPair rc4, boolean [] features){
		XYPair rc21=new XYPair(rc1.x, rc1.y), rc22=new XYPair(rc2.x, rc2.y),
			rc23=new XYPair(rc3.x, rc3.y), rc24=new XYPair(rc4.x, rc4.y);
		int est=0;
		Dimension d=estimateGetWarpProgress(s, uv1, uv2, uv3, uv4, rc21, rc22, rc23, rc24);	
		est+=d.height*d.width;
		
		if(features[0])
			est+=estimateGetGoodWarpProgress(s, t, uv1, uv2, uv3, uv4, rc1, rc2, rc3, rc4);
		
		if(features[1]){
			int tx = Math.max(rc21.x-rc1.x, 0); //origin x of the target image
			int ty = Math.max(rc21.y-rc1.y, 0); //origin y of the target image
			int wx = Math.max(rc1.x-rc21.x, 0); //origin x of the warped image
			int wy = Math.max(rc1.y-rc21.y, 0); //origin y of the warped image
		
			// Bounded area of the intersection
			int xMin = Math.max(tx, wx); 
			int yMin = Math.max(ty, wy);
			int xMax = Math.max(tx, wx)+Math.min(d.width-rc21.x, t.getWidth()-rc1.x)+Math.min(rc21.x, rc1.x);
			int yMax = Math.max(ty, wy)+Math.min(d.height-rc21.y, t.getHeight()-rc1.y)+Math.min(rc21.y,rc1.y);
	
			est+=estimateGetSmallestDistProgress(t, xMin-tx, xMax-tx, yMin-ty, yMax-ty);
			est+=estimateGetSmallestDistProgress(d, xMin-wx, xMax-wx, yMin-wy, yMax-wy);
			est+=(xMax-xMin)*(yMax-yMin);
		}
		return est;
	}
	
	/**
	 * Calculates the distances from every pixel in the intersection region to their nearest edges.
	 * @param im Image to use.
	 * @param xMin Minimum x coordinate of the inersection region.
	 * @param xMax Maximum x coordinate of the inersection region.
	 * @param yMin Minimum y coordinate of the inersection region.
	 * @param yMax Maximum y coordinate of the inersection region.
	 * @return 2D array holding the distance of each pixel cooresponding to an entry in the array
	 * 			to the nearest edge in the image 'im'.
	 */
	private static double [][] getSmallestDist(BufferedImage im, int xMin, int xMax, int yMin, int yMax){
		double [][] res=new double[yMax-yMin][xMax-xMin];
		java.util.Vector e=new java.util.Vector();
		int w=im.getWidth(), h=im.getHeight();
		boolean setProg=false;
		
		for(int x=0; x<w; x++){
			for(int y=0; y<h; y++){
				if(im.getRGB(x, y)!=0 && (x==0 || y==0 || x==w-1 || y==h-1 || 
					im.getRGB(x-1, y)==0 || im.getRGB(x+1, y)==0 || im.getRGB(x, y-1)==0 ||
					im.getRGB(x, y+1)==0)){
					if(xMin<=x && x<xMax && yMin<=y && y<yMax)
						res[y-yMin][x-xMin]=1;
					e.add(new XYPair(x-xMin, y-yMin));
				}else if(im.getRGB(x, y)!=0 && xMin<=x && x<xMax && yMin<=y && y<yMax)
					res[y-yMin][x-xMin]=0;
				else if(xMin<=x && x<xMax && yMin<=y && y<yMax)
					res[y-yMin][x-xMin]=-1;

				prog.getSetCurrentProgress(prog.getSetCurrentProgress(-1)+1);
			}
		}
		XYPair xy;
		double tmp;
		for(int i=0; i<yMax-yMin; i++){
			for(int j=0; j<xMax-xMin; j++){
				if(res[i][j]!=0) continue;
				for(int k=0; k<e.size(); k++){
					xy=(XYPair)e.elementAt(k);
					tmp=(xy.x-j)*(xy.x-j)+(xy.y-i)*(xy.y-i)+1;
					if(res[i][j]==0 || res[i][j]>tmp)
						res[i][j]=tmp;
				}
				prog.getSetCurrentProgress(prog.getSetCurrentProgress(-1)+1);
			}
		}
		
		return res;
	}
	
	/**
	 * Estmates the progress of getSmallestDist.
	 * @param im Image to use.
	 * @param xMin Minimum x coordinate of the inersection region.
	 * @param xMax Maximum x coordinate of the inersection region.
	 * @param yMin Minimum y coordinate of the inersection region.
	 * @param yMax Maximum y coordinate of the inersection region.
	 * @return The estimates total progress of getSmallestDist.
	 */
	private static int estimateGetSmallestDistProgress(BufferedImage im, int xMin, int xMax, int yMin, int yMax){
		return im.getWidth()*im.getHeight()+(yMax-yMin)*(xMax-xMin);
	}
	
	/**
	 * Estmates the progress of getSmallestDist.
	 * @param d Dimension of the image.
	 * @param xMin Minimum x coordinate of the inersection region.
	 * @param xMax Maximum x coordinate of the inersection region.
	 * @param yMin Minimum y coordinate of the inersection region.
	 * @param yMax Maximum y coordinate of the inersection region.
	 * @return The estimates total progress of getSmallestDist.
	 */
	private static int estimateGetSmallestDistProgress(Dimension d, int xMin, int xMax, int yMin, int yMax){
		return d.width*d.height+(yMax-yMin)*(xMax-xMin);
	}
	
	/**
	 * Calculate Bilinear Interpolation to determine a pixel given 4 coordinates surrounding it.
	 * @param img The image the pixel is calculated from.
	 * @param x1 The x coordinate left of the pixel.
	 * @param x2 The x coordinate right of the pixel.
	 * @param y1 The y coordinate below the pixel.
	 * @param y2 The y coordinate above the pixel.
	 * @param x The x coordinate of the desired pixel.
	 * @param y The y coordinate of the desired pixel.
	 * @return The deteremined pixel or 0 when it does not exist.
	 */
	public static int biInt(BufferedImage img, int x1, int x2, int y1, int y2, double x, double y){
	
		int w = img.getWidth();
		int h = img.getHeight();
		
		//check if the coordinates exist in the image, if not, return 0 for the pixel
		if (x1<0 || y1<0 || x2>=w || y2>=h)
			return 0;
	
		double weightx = 1-((x2-x)/(x2-x1)); // the weight costs 0 to 1 depending on the distance between x1 and x2
		double weighty = 1-((y2-y)/(y2-y1)); // the weight costs 0 to 1 depending on the distance between y1 and y2
		
		// a is the top left pixel
		// b is the top right pixel
		// c is the bottom left pixel
		// d is the bottom right pixel
		//System.out.println("x1="+x1+", x2="+x2+", y1="+y1+", y2="+y2);
		int pixela = img.getRGB(x1,y2);
		int pixelb = img.getRGB(x2,y2);
		int pixelc = img.getRGB(x1,y1);
		int pixeld = img.getRGB(x2,y1);
		
		//I(x_1,y_2)
		int alphaa = (pixela >> 24) & 0xff;
		int reda = (pixela >> 16) & 0xff; 
		int greena = (pixela >> 8) & 0xff;
		int bluea = (pixela) & 0xff;
		
		//I(x_2,y_2)
		int alphab = (pixelb >> 24) & 0xff;
		int redb = (pixelb >> 16) & 0xff; 
		int greenb = (pixelb >> 8) & 0xff;
		int blueb = (pixelb) & 0xff;
		
		//I(x_1,y_1)
		int alphac = (pixelc >> 24) & 0xff;
		int redc = (pixelc >> 16) & 0xff; 
		int greenc = (pixelc >> 8) & 0xff;
		int bluec = (pixelc) & 0xff;
		
		//I(x_2,y_1)
		int alphad = (pixeld >> 24) & 0xff;
		int redd = (pixeld >> 16) & 0xff; 
		int greend = (pixeld >> 8) & 0xff;
		int blued = (pixeld) & 0xff;
		
		//I(x,y)=(sig_y*sig_x*I(x_2,x_2))+(sig_y*(1-sig_x)*I(x_1,y_2))+((1-sig_y)*sig_x*I(x_2,y_1))+((1-sig_y)*(1-sig_x)*I(x_1,y_1))
		int alphaxy = (int)((weighty*weightx*alphab) + (weighty*(1-weightx)*alphaa) + ((1-weighty)*weightx*alphad) + ((1-weighty)*(1-weightx)*alphac));
		int redxy = (int)((weighty*weightx*redb) + (weighty*(1-weightx)*reda) + ((1-weighty)*weightx*redd) + ((1-weighty)*(1-weightx)*redc));
		int greenxy = (int)((weighty*weightx*greenb) + (weighty*(1-weightx)*greena) + ((1-weighty)*weightx*greend) + ((1-weighty)*(1-weightx)*greenc));
		int bluexy = (int)((weighty*weightx*blueb) + (weighty*(1-weightx)*bluea) + ((1-weighty)*weightx*blued) + ((1-weighty)*(1-weightx)*bluec));
		
		int pixelxy = (alphaxy << 24) + (redxy << 16) + (greenxy << 8) + bluexy; //shifts
		
		return pixelxy;
	}

	/** Returns the color channel of the image specified by 'colour'.
	 * @param bimg Image to separate.
	 * @param colour "red", "green", or "blue".
	 * @return An image with only the specified channel set using the channel from 'bimg'.
	 */
	public static BufferedImage imageSeperation(BufferedImage bimg, String colour) {
		args.clear();
		args.put("bimg", bimg);
		args.put("colour", colour);
		
		if(prog.getSetCurrentProgress(-1)>0)
			return imageSeperation();
		
		prog.getSetCurrentProgress(1);
		int total=estimateImageSeparationProgress(bimg);
		
		Thread t=new Thread(new Runnable(){
			public void run(){
				result=imageSeperation();
				prog.getSetCurrentProgress(0);
			}
		});
		t.start();
		Dialogs.showProgressBar(prog, total);
		return (BufferedImage)result;
		
	}

	/** Returns the color channel of the image specified by 'colour'.
	 * @return An image with only the specified channel set using the channel from 'bimg'.
	 */
	private static BufferedImage imageSeperation(){
		BufferedImage bimg, img=(BufferedImage)args.get("bimg");
		String colour=(String)args.get("colour");
		bimg=new BufferedImage(img.getWidth(), img.getHeight(), BufferedImage.TYPE_INT_ARGB);
		
		int w = bimg.getWidth(null), h = bimg.getHeight(null);

		for (int x = 0; x < w; x++) {
			for (int y = 0; y < h; y++) {
				int pixel = img.getRGB(x, y); // Colour value at coordinate.
				// Alpha is also in there
				int alpha = (pixel >> 24) & 0xff; //each of these shifts (
				// ,8,16,24) locate the actual
				// value of
				// blue,green,red,alpha
				int red = (pixel >> 16) & 0xff; //0xff removes the previous 8
				// bits to look at desired
				// location
				int green = (pixel >> 8) & 0xff;
				int blue = (pixel) & 0xff;

				// pulls out the specified colour
				if (colour.equals("red")) {
					green = 0;
					blue = 0;
				} else if (colour.equals("green")) {
					red = 0;
					blue = 0;
				} else if(colour.equals("blue")){
					red = 0;
					green = 0;
				}
				pixel = (alpha << 24) + (red << 16) + (green << 8) + blue; //shifts
				// back
				// the
				// proper
				// bits
				bimg.setRGB(x, y, pixel);
				prog.getSetCurrentProgress(prog.getSetCurrentProgress(-1)+1);
			}
		}
		return bimg;
	}
	
	/**
	 * Estimates the total progress for channel separation.
	 * @param bimg Image to separate.
	 * @return Estimate of total progress.
	 */
	public static int estimateImageSeparationProgress(BufferedImage bimg){
		return bimg.getHeight()*bimg.getWidth();
	}

	/** Changes and returns the greyscaled version of an image.
	 * @param bimg The image.
	 * @return Greyscaled image.
	 */
	public static BufferedImage convertGreyscale(BufferedImage bimg) {
		args.clear();
		args.put("bimg", bimg);

		if(prog.getSetCurrentProgress(-1)>0)
			return convertGreyscale();
		
		prog.getSetCurrentProgress(1);
		int total=estimateConvertGreyscaleProgress(bimg);
		
		Thread t=new Thread(new Runnable(){
			public void run(){
				result=convertGreyscale();
				prog.getSetCurrentProgress(0);
			}
		});
		t.start();
		Dialogs.showProgressBar(prog, total);
		return (BufferedImage)result;
		
	}
	/** Changes and returns the greyscaled version of an image.
	 * @return Greyscaled image.
	 */
	public static BufferedImage convertGreyscale() {
		BufferedImage img=(BufferedImage)args.get("bimg"),
			bimg=new BufferedImage(img.getWidth(), img.getHeight(), BufferedImage.TYPE_INT_ARGB);
		
		int w = bimg.getWidth(null), h = bimg.getHeight(null);
		for (int x = 0; x < w; x++) {
			for (int y = 0; y < h; y++) {
				// Go through every pixel and take the weighted average of the RGB components.
				int pixel = img.getRGB(x, y); // Colour value at coordinate.
				// Alpha is also in there
				int alpha = (pixel >> 24) & 0xff; //each of these shifts (
				// ,8,16,24) locate the actual
				// value of
				// blue,green,red,alpha
				int red = (pixel >> 16) & 0xff; //0xff removes the previous 8
				// bits to look at desired
				// location
				int green = (pixel >> 8) & 0xff;
				int blue = (pixel) & 0xff;

				// convert every pixel to greyscale by using the weighted
				// average value of the RGB components
				double greyscale = (red * 0.5) + (green * 0.3) + (blue * 0.2);

				red = (int) greyscale;
				green = (int) greyscale;
				blue = (int) greyscale;

				pixel = (alpha << 24) + (red << 16) + (green << 8) + blue; //shifts
				// back
				// the
				// proper
				// bits
				bimg.setRGB(x, y, pixel);
				prog.getSetCurrentProgress(prog.getSetCurrentProgress(-1)+1);
			}
		}
		return bimg;
	}
	
	/**
	 * Estimates the totl progress for greying an image.
	 * @param bimg Image to greyscale.
	 * @return Total progress to greyscale an image.
	 */
	public static int estimateConvertGreyscaleProgress(BufferedImage bimg){
		return bimg.getWidth()*bimg.getHeight();
	}

	/** Returns the Gaussian smoothed image.
	 * @param frame The container Panel of the GUI application.
	 * @param img The image.
	 * @param sigma Sigma used to compute the size of the mask and the mask values.
	 * 					If it's < 0 then user will be prompted for the sigma value.
	 * @return A Gaussian smoothed image or null if user was prompted
	 * 				for sigma and "cancel" was pressed.
	 */
	public static BufferedImage getGaussian(Component frame, BufferedImage img, double sigma) {
		args.clear();
		String s;
		// Keep asking the user for the sigma until a valid value is entered.
		while (sigma<0) {
			s = (String) JOptionPane.showInputDialog(frame,
					"Enter the sigma value to be\n used in the Gaussian mask:",
					"Sigma", JOptionPane.PLAIN_MESSAGE, null, null, "1");
					
			if(s==null) return null;
			
			try {
				sigma = Double.parseDouble(s);
				break;
			} catch (Exception e) {
			}
		}
		args.put("img", img);
		args.put("sigma", new Double(sigma));
		
		if(prog.getSetCurrentProgress(-1)>0)
			return getGaussian();
		
		prog.getSetCurrentProgress(1);
		int total=estimateGetGaussianProgress(img, sigma);
		
		Thread t=new Thread(new Runnable(){
			public void run(){
				result=getGaussian();
				prog.getSetCurrentProgress(0);
			}
		});
		t.start();
		Dialogs.showProgressBar(prog, total);
		return (BufferedImage)result;
	}
	
	/** Returns the Gaussian smoothed image.
	 * @return A Gaussian smoothed image or null if user was prompted
	 * 				for sigma and "cancel" was pressed.
	 */
	private static BufferedImage getGaussian() {
		BufferedImage img=(BufferedImage)args.get("img");
		double sigma=((Double)args.get("sigma")).doubleValue();
		
		BufferedImage gaus= new BufferedImage(img.getWidth(null), img.getHeight(null), BufferedImage.TYPE_INT_ARGB);
		
		// determine the size of the mask (must be odd).
		int size = (int) Math.ceil(sigma * 6);
		if(size<=1){
			gaus.createGraphics().drawImage(img, 0, 0, null);
			return gaus;
		}
		if (size % 2 == 0)
			size++;

		double[][] mask = new double[size][size];
		
		// Fill the mask array.
		for (int i = 0; i < size; i++) {
			for (int j = 0; j < size; j++) {
				mask[i][j] = findGaussian(j - size / 2, size / 2 - i, sigma);
				prog.getSetCurrentProgress(prog.getSetCurrentProgress(-1)+1);
			}
		}
		
		int pixel, ax, ay;
		double r, g, b, a;

		// Apply the mask to every pixel of the image.
		for (int x = 0; x < img.getWidth(null); x++) {
			for (int y = 0; y < img.getHeight(null); y++) {
				r=g=b=a = 0;
				// Apply the mask centered at this pixel.
				for (int i = 0; i < size; i++) {
					for (int j = 0; j < size; j++) {
						ax=Math.max(0, Math.min(img.getWidth(null)-1, x + j - size / 2));
						ay=Math.max(0, Math.min(img.getHeight(null)-1, y + i - size / 2));
						pixel=img.getRGB(ax, ay);
						a+=((pixel>>24)&0xff)*mask[i][j];
						r+=((pixel>>16)&0xff)*mask[i][j];
						g+=((pixel>>8)&0xff)*mask[i][j];
						b+=((pixel)&0xff)*mask[i][j];
						prog.getSetCurrentProgress(prog.getSetCurrentProgress(-1)+1);
					}
				}
				gaus.setRGB(x, y, ((int)a<<24) + ((int)r<<16) + ((int)g<<8)+(int)b);
			}
		}
		return gaus;
	}
	
	/**
	 * Estimates the total progress for smoothing an image.
	 * @param img Image to smooth.
	 * @param sigma Sigma to use when smoothing an image.
	 * @return Total estimated progress for smoothing.
	 */
	public static int estimateGetGaussianProgress(BufferedImage img, double sigma){
		// determine the size of the mask (must be odd).
		int size = (int) Math.ceil(sigma * 6);
		if(size<=1){
			return 2;
		}
		if (size % 2 == 0)
			size++;

		return size*size+img.getHeight()*img.getWidth()*size*size;
	}
	
	/** The Gaussian function for filling the Gaussian mask.
	 * @param x The x coordinate in a mask.
	 * @param y The y coordinate in a mask.
	 * @param sigma The sigma to be used for the function.
	 * @return The result of applying the function with the above parameters.
	 */
	private static double findGaussian(int x, int y, double sigma){
		double t1 = 1 / (2 * Math.PI * sigma * sigma);
		
		double t2 = Math.exp( - ( (y*y) + (x*x) ) / (2 * sigma * sigma));  //m assumed to be 0 in Gaussian formula
		
		return t1 * t2;
	}

	/** Returns the 2D array representing the LoG of an image.
	 * @param frame The container Panel of the GUI application.
	 * @param img The image.
	 * @param sigma Sigma used to compute the size of the mask and the mask values.
	 * 					If it's < 0 then user will be prompted for the sigma value.
	 * @return A 2D array representing the LoG of the image or null if user was prompted
	 * 				for sigma and "cancel" was pressed.
	 */
	public static double[][] getLoG(Component frame, BufferedImage img, double sigma) {
		String s;
		// Keep asking the user for the sigma until a valid value is entered.
		while (sigma<0) {
			s = (String) JOptionPane.showInputDialog(frame,
					"Enter the sigma value to be\n used in the Gaussian mask:",
					"Sigma", JOptionPane.PLAIN_MESSAGE, null, null, "1");
					
			if(s==null) return null;
			
			try {
				sigma = Double.parseDouble(s);
				break;
			} catch (Exception e) {
			}
		}
		args.clear();
		args.put("img", img);
		args.put("sigma", new Double(sigma));
		
		if(prog.getSetCurrentProgress(-1)>0){
			return getLoG();
		}
		
		prog.getSetCurrentProgress(1);
		int total=estimateGetLoGProgress(img, sigma);
		
		Thread t=new Thread(new Runnable(){
			public void run(){
				result=getLoG();
				prog.getSetCurrentProgress(0);
			}
		});
		t.start();
		Dialogs.showProgressBar(prog, total);
		return (double [][])result;
		
	}

	/** Returns the 2D array representing the LoG of an image.
	 * @return A 2D array representing the LoG of the image or null if user was prompted
	 * 				for sigma and "cancel" was pressed.
	 */
	private static double[][] getLoG() {
		BufferedImage img=(BufferedImage)args.get("img");
		double sigma=((Double)args.get("sigma")).doubleValue();
		img=convertGreyscale(img);

		// determine the size of the mask (must be odd).
		int size = (int) Math.ceil(sigma * 6);
		if(size<3){
			size=3;
			sigma=((double)size)/6;
		}else if (size % 2 == 0)
			size++;
		
		double[][] mask = new double[size][size];
		
		// Fill the mask array.
		for (int i = 0; i < size; i++) {
			for (int j = 0; j < size; j++) {
				mask[i][j] = findLoG(j - size / 2, size / 2 - i, sigma);
				prog.getSetCurrentProgress(prog.getSetCurrentProgress(-1)+1);
			}
		}

		double[][] log = new double[img.getHeight(null)][img.getWidth(null)];
		int pixel, ax, ay;
		double alpha;

		// Apply the mask to every pixel of the image.
		for (int x = 0; x < img.getWidth(null); x++) {
			for (int y = 0; y < img.getHeight(null); y++) {
				log[y][x] = 0;
				// Apply the mask centered at this pixel.
				for (int i = 0; i < size; i++) {
					for (int j = 0; j < size; j++) {
						ax=Math.max(0, Math.min(img.getWidth(null)-1, x + j - size / 2));
						ay=Math.max(0, Math.min(img.getHeight(null)-1, y + i - size / 2));
						pixel=img.getRGB(ax, ay);
						alpha=(pixel>>24)&0xff;
						log[y][x] += mask[i][j] * alpha/255*(pixel & 0xff);
						prog.getSetCurrentProgress(prog.getSetCurrentProgress(-1)+1);
					}
				}
			}
		}
		return log;
	}
	
	/**
	 * Estimates total progress for getting the LoG of an image.
	 * @param img Image to get the LoG for.
	 * @param sigma Sigma to use to get the LoG.
	 * @return Estimated total progress to get the LoG.
	 */
	public static int estimateGetLoGProgress(BufferedImage img, double sigma){
		// determine the size of the mask (must be odd).
		int size = (int) Math.ceil(sigma * 6);
		if(size<3){
			size=3;
			sigma=((double)size)/6;
		}else if (size % 2 == 0)
			size++;

		return estimateConvertGreyscaleProgress(img)+size*size+img.getHeight()*img.getWidth()*size*size;
	}

	/** The Laplacian of a Gaussian function for filling the LoG mask.
	 * @param x The x coordinate in a mask.
	 * @param y The y coordinate in a mask.
	 * @param sigma The sigma to be used for the function.
	 * @return The result of applying the function with the above parameters.
	 */
	private static double findLoG(int x, int y, double sigma) {
		double t3 = Math.exp(-(x * x + y * y) / (2 * sigma * sigma));

		double t2 = 1 - (x * x + y * y) / (2 * sigma * sigma);

		double t1 = -1 / (Math.PI * Math.pow(sigma, 4));

		return t1 * t2 * t3;
	}

	/** Returns the 2D array representing the zero crossings in the LoG of an image.
	 * @param img Image to get zero-crossings for.
	 * @return 2D array representing the zero crossings in the LoG of an image.
	 */
	public static int[][] getZeroXings(Component frame, BufferedImage img, double sigma) {
		String s;
		// Keep asking the user for the sigma until a valid value is entered.
		while (sigma<0) {
			s = (String) JOptionPane.showInputDialog(frame,
					"Enter the sigma value to be\n used in the Gaussian mask:",
					"Sigma", JOptionPane.PLAIN_MESSAGE, null, null, "1");
					
			if(s==null) return null;
			
			try {
				sigma = Double.parseDouble(s);
				break;
			} catch (Exception e) {
			}
		}
		args.clear();
		args.put("img", img);
		args.put("sigma", new Double(sigma));
		
		if(prog.getSetCurrentProgress(-1)>0)
			return getZeroXings();
		
		prog.getSetCurrentProgress(1);
		int total=estimateGetZeroXingsProgress(img, sigma);
		
		Thread t=new Thread(new Runnable(){
			public void run(){
				result=getZeroXings();
				prog.getSetCurrentProgress(0);
			}
		});
		t.start();
		Dialogs.showProgressBar(prog, total);
		return (int[][])result;
	}

	/** Returns the 2D array representing the zero crossings in the LoG of an image.
	 * @param log 2D array representing the LoG of an image.
	 * @return 2D array representing the zero crossings in the LoG of an image.
	 */
	private static int[][] getZeroXings(){
		BufferedImage img=(BufferedImage)args.get("img");
		double sigma=((Double)args.get("sigma")).doubleValue();
		double [][] log=getLoG(null, img, sigma);
		int[][] zx = new int[log.length][log[0].length];

		// initialize all the entries.
		for (int i = 0; i < log.length; i++) {
			for (int j = 0; j < log[0].length; j++) {
				zx[i][j] = 0;
				prog.getSetCurrentProgress(prog.getSetCurrentProgress(-1)+1);
			}
		}
		
		// For each pixel determine if there's a zero-crossing.
		for (int i = 0; i < log.length - 1; i++) {
			for (int j = 0; j < log[0].length - 1; j++) {
				// See if there's a 0-crossing in the column.
				if (log[i][j] > 0 && log[i + 1][j] < 0 || log[i][j] < 0
						&& log[i + 1][j] > 0) {
					// Choose the smaller value to be an edge pixel.
					if (Math.abs(log[i][j]) > Math.abs(log[i + 1][j]))
						zx[i + 1][j] = 255;
					else
						zx[i][j] = 255;
				}
				// See if there's a 0-crossing in the row.
				if (log[i][j] > 0 && log[i][j + 1] < 0 || log[i][j] < 0
						&& log[i][j + 1] > 0) {
					// Choose the smaller value to be an edge pixel.
					if (Math.abs(log[i][j]) > Math.abs(log[i][j + 1]))
						zx[i][j + 1] = 255;
					else
						zx[i][j] = 255;
				}
				prog.getSetCurrentProgress(prog.getSetCurrentProgress(-1)+1);
			}
		}
		return zx;
	}
	
	/**
	 * Estimates total progress for finding zero-crossings in an image.
	 * @param img Image to use.
	 * @param sigma Sigma to use to find the zero-crossings.
	 * @return Estimated total progress for finding zero-crossings.
	 */
	public static int estimateGetZeroXingsProgress(BufferedImage img, double sigma){
		return estimateGetLoGProgress(img, sigma)+2*img.getWidth()*img.getHeight();
	}

	/** Computes the gradient image using the Sobel mask.
	 * @param bi Image to compute the gradient for.
	 * @return 2D array of gradient vectors for the image.
	 */
	public static GradVector[][] getGradient(BufferedImage bi) {
		args.clear();
		args.put("bi", bi);
		
		if(prog.getSetCurrentProgress(-1)>0)
			return getGradient();
		
		prog.getSetCurrentProgress(1);
		int total=estimateGetGradientProgress(bi);
		
		Thread t=new Thread(new Runnable(){
			public void run(){
				result=getGradient();
				prog.getSetCurrentProgress(0);
			}
		});
		t.start();
		Dialogs.showProgressBar(prog, total);
		return (GradVector[][])result;
	}

	/** Computes the gradient image using the Sobl mask.
	 * @return 2D array of gradient vectors for the image.
	 */
	private static GradVector[][] getGradient() {
		BufferedImage bi=(BufferedImage)args.get("bi");
		int w = bi.getWidth(null), h = bi.getHeight(null);
		GradVector[][] grad = new GradVector[h][w];
		
		bi=convertGreyscale(bi);

		// go through all columns
		for (int i = 0; i < h; i++) {
			// go through all rows
			for (int j = 0; j < w; j++) {
				int p = bi.getRGB(j, i);
				p = p & 0xff;

				// horizontal component.
				int pr = -(bi.getRGB(Math.max(j - 1, 0), Math.max(0, i - 1)) & 0xff) 
						- 2 * (bi.getRGB(Math.max(j - 1, 0), i) & 0xff)
						- (bi.getRGB(Math.max(j - 1, 0), Math.min(i + 1, h-1)) & 0xff)
						+ (bi.getRGB(Math.min(j + 1, w-1), Math.max(0, i - 1)) & 0xff)
						+ 2 * (bi.getRGB(Math.min(j + 1, w-1), i) & 0xff)
						+ (bi.getRGB(Math.min(j + 1, w-1), Math.min(i + 1, h-1)) & 0xff);

				// vertical component.
				int pc = - (bi.getRGB(Math.max(0, j - 1), Math.min(i + 1, h-1)) & 0xff) - 2
						* (bi.getRGB(j, Math.min(i + 1, h-1)) & 0xff)
						- (bi.getRGB(Math.min(j + 1, w-1), Math.min(i + 1, h-1)) & 0xff)
						+ (bi.getRGB(Math.max(0, j - 1), Math.max(0, i - 1)) & 0xff) + 2
						* (bi.getRGB(j, Math.max(0, i - 1)) & 0xff)
						+ (bi.getRGB(Math.min(j + 1, w-1), Math.max(0, i - 1)) & 0xff);

				grad[i][j] = new GradVector(pr, pc, true);
				prog.getSetCurrentProgress(prog.getSetCurrentProgress(-1)+1);
			}
		}
		return grad;
	}

	/**
	 * Estimates total progress needed for getting gradient of an image.
	 * @param img Image to get the gradient for.
	 * @return Estimated total progress to get the gradient.
	 */
	public static int estimateGetGradientProgress(BufferedImage img){
		return estimateConvertGreyscaleProgress(img)+img.getHeight()*img.getWidth();
	}
}