import java.awt.*;

/** Shape class for the warp-box.
 */
public class WarpRect extends Shape {
	double rad;		// radius of circles on the corners of the box.
	XYPair [] xys;	// corners of the box.
	
	/**
	 * Constructor:
	 * @param p1 1st corner.
	 * @param p2 2nd corner.
	 * @param p3 3rd corner.
	 * @param p4 4th corner.
	 * @param c Color of the box.
	 * @param rad radius of circles on the corners.
	 */
	public WarpRect(XYPair p1, XYPair p2, XYPair p3, XYPair p4, Color c, int rad){
		super(c);
		pos.add(p1);
		pos.add(p2);
		pos.add(p3);
		pos.add(p4);
		this.rad=rad;
		xys=new XYPair[4];
		xys[0]=p1;
		xys[1]=p2;
		xys[2]=p3;
		xys[3]=p4;
	}
	
	/** Paints this circle.
	 * @param g Graphics object to paint on.
	 */
	public void paint(Graphics g){
		Color c=g.getColor();
		g.setColor(color);
		
		// draw the line between 1st and 2nd corners.
		double dist=Math.sqrt(Math.pow(xys[0].x-xys[1].x, 2)+Math.pow(xys[0].y-xys[1].y, 2));
		int dx=(int)Math.round(rad/dist*(xys[1].x-xys[0].x));
		int dy=(int)Math.round(rad/dist*(xys[1].y-xys[0].y));
		g.drawLine(xys[0].x+dx, xys[0].y+dy, xys[1].x-dx, xys[1].y-dy);
		
		// draw the line between 2nd and 3rd corners.
		dist=Math.sqrt(Math.pow(xys[1].x-xys[2].x, 2)+Math.pow(xys[1].y-xys[2].y, 2));
		dx=(int)Math.round(rad/dist*(xys[2].x-xys[1].x));
		dy=(int)Math.round(rad/dist*(xys[2].y-xys[1].y));
		g.drawLine(xys[1].x+dx, xys[1].y+dy, xys[2].x-dx, xys[2].y-dy);
		
		// draw the line between 3rd and 4th corners.
		dist=Math.sqrt(Math.pow(xys[2].x-xys[3].x, 2)+Math.pow(xys[2].y-xys[3].y, 2));
		dx=(int)Math.round(rad/dist*(xys[3].x-xys[2].x));
		dy=(int)Math.round(rad/dist*(xys[3].y-xys[2].y));
		g.drawLine(xys[2].x+dx, xys[2].y+dy, xys[3].x-dx, xys[3].y-dy);
		
		// draw the line between 4th and 1st corners.
		dist=Math.sqrt(Math.pow(xys[0].x-xys[3].x, 2)+Math.pow(xys[0].y-xys[3].y, 2));
		dx=(int)Math.round(rad/dist*(xys[0].x-xys[3].x));
		dy=(int)Math.round(rad/dist*(xys[0].y-xys[3].y));
		g.drawLine(xys[3].x+dx, xys[3].y+dy, xys[0].x-dx, xys[0].y-dy);
		g.setColor(c);
	}
}
